#! /usr/bin/env bash

source util.sh

VERSION=$1
PREFIX="$2"
URL="https://ftp.gnu.org/gnu/binutils/binutils-$VERSION.tar.xz"
ARCHIVE=$(basename "$URL")

# Avoid rebuilds of the same version

existing_as="$PREFIX/bin/sh-elf-as"

if [[ -f "$existing_as" ]]; then
  existing_version=$($existing_as --version | head -n 1 | grep -Eo '[0-9.]+$')
  if [[ $existing_version == $VERSION ]]; then
    echo "$TAG Version $VERSION already installed, skipping rebuild"
    if [[ -e build ]]; then
      rm -rf build
    fi
    exit 0
  fi
fi

# Aggressive parameter to avoid rebuilds

if [[ ! -z "$ACCEPT_ANY" ]]; then
  if command -v sh-elf-as >/dev/null 2>&1; then
    echo "$TAG Found sh-elf-as in PATH and ACCEPT_ANY is set, skipping build"
    exit 0
  fi
fi

# Check dependencies for binutils and GCC

if command -v apt >/dev/null 2>&1; then
  deps="libmpfr-dev libmpc-dev libgmp-dev libpng-dev libppl-dev flex g++ git texinfo xz-utils"
  pm=apt
  pm_has="dpkg -s"
  pm_install="sudo apt install"
elif command -v pacman >/dev/null 2>&1; then
  deps="mpfr libmpc gmp libpng ppl flex gcc git texinfo xz"
  pm=pacman
  pm_has="pacman -Qi"
  pm_install="sudo pacman -S"
else
  trust_deps=1
fi

missing=""
if [[ -z "$trust_deps" ]]; then
  for d in $deps; do
    if ! $pm_has $d >/dev/null 2>&1; then
      missing="$missing $d"
    fi
  done
fi

# Offer to install dependencies

if [[ ! -z "$missing" ]]; then
  echo "$TAG Based on $pm, some dependencies are missing: $missing"
  echo -n "$TAG Do you want to run '$pm_install $missing' to install them (Y/n)? "

  read do_install
  if [[ "$do_install" == "y" || "$do_install" == "Y" || "$do_install" == "" ]]; then
    $pm_install $missing
  else
    echo "$TAG Skipping dependencies, hoping it will build anyway."
  fi
fi

# Download archive

if [[ -f "$ARCHIVE" ]]; then
  echo "$TAG Found $ARCHIVE, skipping download"
else
  echo "$TAG Downloading $URL..."
  if command -v curl >/dev/null 2>&1; then
    curl $URL -o $ARCHIVE
  elif command -v wget >/dev/null 2>&1; then
    wget -q --show-progress $URL -O $ARCHIVE
  else
    echo "$TAG error: no curl or wget; install one or download archive yourself" >&2
    exit 1
  fi
fi

# Extract archive

echo "$TAG Extracting $ARCHIVE..."
tar -xJf $ARCHIVE

# Create build folder

[[ -d "build" ]] && rm -rf build
mkdir build

# Configure. binutils does not support the uninstall target (wow) so we just
# install in this directory and later symlink executables to $PREFIX/bin.

PREFIX="$(pwd)"
cd build

echo "$TAG Configuring binutils..."
run_quietly giteapc-configure.log \
../binutils-$VERSION/configure --prefix="$PREFIX" --target=sh3eb-elf --with-multilib-list=m3,m4-nofpu --program-prefix=sh-elf- --enable-libssp --enable-lto
